#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import logging
import codecs
import shutil
import hashlib
from jinja2 import Environment
from jinja2 import FileSystemLoader



class Operation():
    def __init__(self, **kwargs):

        self.head = kwargs.get("head", None)
        self.leaf_info = kwargs.get("leaf_info", None)
        self.documentation = kwargs.get("documentation", None)
        self.example = kwargs.get("example", None)
        self.namespace = kwargs.get("namespace", None)
        self.xml_tail = kwargs.get("xml_tail", None)
        self.head_content = kwargs.get("head_content", None)
        self.output_file_path = kwargs.get("output_file_path", None)
        self.filename = kwargs.get("filename", None)
        self.argument_spec = kwargs.get("argument_spec", None)
        self.key_list = kwargs.get("key_list", None)
        self.module = kwargs.get("module", None)
        self.user_check_stmts = kwargs.get("user_check_stmts", None)

    def gen_script(self):
        file_name = str(self.filename) + '.py'
        result = os.path.join(self.output_file_path, file_name)
        env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'templates'), 'utf-8'),
                          auto_reload=True)
        template = env.get_template('module.html')
        kwarg = dict(head=self.head,
                     documentation=self.documentation,
                     example=self.example,
                     module=self.module,
                     user_check_stmts=self.user_check_stmts,
                     argument_spec=self.argument_spec,
                     key_list=self.key_list,
                     leaf_info=self.leaf_info,
                     namespaces=self.namespace,
                     xml_head=self.head_content[0] if self.head_content else None,
                     business_tag=self.head_content[1] if self.head_content else None,
                     #comm_module_path=os.path.join(self.output_file_path[1], 'common_module'),
                     message_type=self.head_content[2] if self.head_content else None,
                     xml_tail=self.xml_tail,
                     filename=self.filename,
                     )
        data = template.render(**kwarg)

        # Automatically generate ansible script file
        try:
            if sys.version < '3':
                handle = codecs.open(result, mode="w", encoding="utf-8")
            else:
                handle = open(result, mode="w", encoding="UTF-8")

            handle.writelines(data + '\n')
        except Exception as error:
            logging.error("write script failed: %s", str(error))

    # Main run function
    def run(self):
        # Output ansible automation to generate scripts to the path specified by the user
        logging.info("gen script:%s begin.",str(self.filename)+".py")
        self.gen_script()
        logging.info("gen script:%s end.", str(self.filename) + ".py")
        # Formatting script
        # os.system('autopep8 --in-place --aggressive --aggressive %s.py' %
        #           (os.path.join(self.output_file_path, self.filename)))
        # logging.info("script:%s has been formatted.",str(self.filename) + ".py")


def deploy(output_file_path):
    create_comm_file(output_file_path, get_files())


def get_files():
    file_set = {'checkparams.py',  'ne_base.py', '__init__.py', 'xml_build_with_xmlns.py'}
    path = os.path.dirname(__file__)
    files = {os.path.join(path, file_name) for file_name in file_set}
    return files


def create_comm_file(output_file_path, files):
    if not (output_file_path and files):
        return
    comm_path = os.path.join(output_file_path, 'common_module')
    for file in files:
        if not os.path.exists(comm_path):
            os.mkdir(comm_path)
            shutil.copy(file, comm_path)
        else:
            multi_user_operation(comm_path, file)


def multi_user_operation(comm_path, file):
    file_list = [i for i in os.listdir(comm_path)]
    if os.path.basename(file) in file_list and cmp_md5_value(comm_path, file):
        return
    else:
        shutil.copy(file, comm_path)


def get_tool_file_md5_set(file_path):
    if not file_path:
        return
    temp_set = set()
    try:
        file = open(file_path, 'rb')
        md5_obj = hashlib.md5()
        md5_obj.update(file.read())
        hash_code = md5_obj.hexdigest()
        file.close()
        temp_set.add(hash_code)
        return temp_set
    except IOError as ex:
        sys.stderr.write("error %s: %s\n" % (file_path, str(ex)))


def get_deploy_file_md5_set(output_path, file_path):
    if not (output_path and file_path):
        return
    temp_set = set()
    try:
        for deploy_file in os.listdir(output_path):
            if os.path.basename(file_path) == deploy_file:
                file = open(os.path.join(output_path, deploy_file), 'rb')
                md5_obj = hashlib.md5()
                md5_obj.update(file.read())
                hash_code = md5_obj.hexdigest()
                file.close()
                temp_set.add(hash_code)
        return temp_set
    except IOError as ex:
        sys.stderr.write("error %s: %s\n" % (output_path, str(ex)))



def cmp_md5_value(output_file_path, file):

    tool_md5_value_set = get_tool_file_md5_set(file)
    deploy_md5_value_set = get_deploy_file_md5_set(output_file_path, file)
    if (tool_md5_value_set - deploy_md5_value_set) or (deploy_md5_value_set - tool_md5_value_set):
        return False
    else:
        return True



#!/usr/bin/env python
# -*- coding: utf-8

try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET

import logging
import sys


# Collation: Sort by the length of xpathNode.
def order_by_xpath(elem):
    for key in elem:
        xpath_result = key.split("/")
    return len(xpath_result)

# remove the prefix from XPATH
# fun called before: [ { '/node1/prefix2:node2/prefix3:node3':[ text3,xmlns3 ] },... ]
# fun called after: [ { '/node1/node2/node3':[ text3,xmlns3,'/node1/prefix2:node2/prefix3:node3' ] },... ]
def xmlns_info_remove_prefix(xmlns_info_list):
    """
    :param xmlns_info_list: the xmlns info list
    """
    new_xmlns_info_list = []
    for item in xmlns_info_list:
        new_dict = {}
        for xapth_key in item:
            xapth_key_list = xapth_key.split("/")[1:]
            new_xapth_key = ''
            for each_node in xapth_key_list:
                new_xapth_key = new_xapth_key + "/" + each_node.split(":")[-1]
            temp_list = item[xapth_key]
            temp_list.append(xapth_key)
        new_dict[new_xapth_key] = temp_list
        new_xmlns_info_list.append(new_dict)
    return new_xmlns_info_list

# Recursive function,to generate xml with xmlns.
def get_all_xpath_from_new_xmlns_info(new_xmlns_info_list):
    """
    :param new_xmlns_info_list:  The xmlns info list generated by "_full.xml".Have been sorted and the key without prefix.
    :return xpath_list: [ '/node1','/node1/node2','/node1/node2/node3',... ]     ( without prefix )
    """
    xpath_list = []
    for item in new_xmlns_info_list:
        for xapth_key in item:
            xpath_list.append(xapth_key)
    return xpath_list

# Recursive function,to generate xml with xmlns.
def create_xml_with_xmlns(root, new_xmlns_info_list, all_xpath_without_prefix, dest_xpath, current_element):
    """
    :param root: The xml root which need to be rebuild.( always come from instance xml )
    :param new_xmlns_info_list: The xmlns info list generated by "_full.xml".Have been sorted and the key without prefix.
    :param all_xpath_without_prefix: get all xpath key from new_xmlns_info_list.Type: list
    :param dest_xpath: record the path of rebuild
    :param current_element: the current generate element root
    """
    if root.getchildren():
        # traversing the xml root which need to be rebuild.( always come from instance xml )
        for child_of_root in root.getchildren():
            dest_xpath = dest_xpath + "/" + child_of_root.tag

            # get value from instance xml
            instance_value = child_of_root.text
            instance_attributes = child_of_root.attrib  # instance_attributes: operation="XXX"

            # instance node'xpath must exist in "_full.xml"
            if dest_xpath in all_xpath_without_prefix:
                original_tag_name = ""
                xmlns = ""
                value = ""

                # find the same xpath from new_xmlns_info_list,get the original_tag_name
                for new_xmlns_info_list_item in new_xmlns_info_list:
                    for xapth_key in new_xmlns_info_list_item:
                        if xapth_key == dest_xpath:
                            tag_name_before = new_xmlns_info_list_item[xapth_key][2]
                            original_tag_name = tag_name_before.split("/")[-1]
                            value = instance_value
                            xmlns = new_xmlns_info_list_item[xapth_key][1]

                #  use the original_tag_name to rebuild sub element node.
                sub_element = ET.SubElement(current_element, original_tag_name)
                # set the element's value,the value come from instance xml
                if value != '':
                    sub_element.text = value
                # set the element's attributes,the attributes come from instance xml.Like: operation="XXX"
                if instance_attributes:
                    for instance_attribute in instance_attributes:
                        sub_element.set(instance_attribute, instance_attributes[instance_attribute])
                #  set the element's attributes,the attributes come from "_full.xml".Like: xmlns="XXX"
                if xmlns:
                    xmlns_list = xmlns.split("@")
                    for xmlns in xmlns_list[1:]:
                        xmlns_temp = xmlns.split("=")
                        xmlns_key = xmlns_temp[0]
                        xmlns_value = xmlns_temp[1].strip('"')
                        sub_element.set(xmlns_key, xmlns_value)
                # recursive
                create_xml_with_xmlns(child_of_root, new_xmlns_info_list, all_xpath_without_prefix, dest_xpath, sub_element)
            else:
                logging.info(dest_xpath+":This xpath not in the full.xml,please check.")
            dest_xpath = dest_xpath[:-(len(child_of_root.tag)+1)]

# join xmlns into fil_content
def xml_parser_join_xmlns(xml_content, xmlns_info_list, filter_or_config_type):
    """
    :param xml_content: the xml str which need to be joined xmlns.
    :param xmlns_info: xmlns info list generated by "_full.xml"
    :param filter_or_config_type: to distinguish filter or config message
    :return:xml_str: xml str which contain the xmlns
    """
    xmlns_info_list.sort(key=order_by_xpath)
    new_xmlns_info_list = xmlns_info_remove_prefix(xmlns_info_list)
    all_xpath_without_prefix = get_all_xpath_from_new_xmlns_info(new_xmlns_info_list)

    original_root = ET.fromstring(xml_content)
    dest_xpath = ""

    if filter_or_config_type == 'filter':
        generate_root = ET.Element('filter')
        generate_root.set("type", "subtree")
    elif filter_or_config_type == 'config':
        generate_root = ET.Element('config')
        generate_root.set("xmlns:nc", "urn:ietf:params:xml:ns:netconf:base:1.0")
    elif filter_or_config_type == 'rpc':
        generate_root = ET.Element('rpc')
    create_xml_with_xmlns(original_root, new_xmlns_info_list, all_xpath_without_prefix, dest_xpath, generate_root)
    if sys.version < "3":
        xml_str = ET.tostring(generate_root, method='xml')
    else:
        xml_str = ET.tostring(generate_root, method='xml', encoding="utf-8").decode('utf-8')
    return xml_str


#!/usr/bin/env python
# -*- coding: utf-8 -*-



import re
import copy
from collections import OrderedDict


class Util(object):
    def __init__(self):
        self.age = -1

    @staticmethod
    def translate_restrict(rests):
        rest_nums = []
        rest_part_pattern = re.compile(r"(\d+\s*\.\.\s*\d+)|(\d+)")
        for src_rest in rests:
            matches = [res_t[0] if res_t[0] else res_t[1] for res_t in rest_part_pattern.findall(src_rest)]
            for res in matches:
                if ".." in res:
                    rng = [int(endpoint.strip()) for endpoint in res.split("..")]
                    rest_nums.append((rng[0], rng[1]))
                else:
                    num = int(res.strip())
                    rest_nums.append((num, num))
        rest_nums.sort(key=lambda item: item[0])

        return rest_nums

    @staticmethod
    def restrict_set_intersec(restricts):
        if not restricts:
            return []

        def get_intersec(t1, t2):
            t1_min = t1[0]
            t1_max = t1[1]
            if(t1_max is None):
                t1_max = t1_min
            t2_min = t2[0]
            t2_max = t2[1]
            if t2_max is None:
                t2_max = t2_min

            if t1_max < t2_min:
                return [[t1_min,t1_max], [t2_min,t2_max]]
            return [(max(t1_min, t2_min), min(t1_max, t2_max))]
        restricts.sort(key=lambda item: item[0])
        result = []
        result.append(restricts[0])
        idx = 1
        while idx < len(restricts):
            _ = get_intersec(result[-1], restricts[idx])
            result = result[:-1] + _
            idx += 1

        return result


def split_to_xpath_and_orderdict(xml_dict):
    tmp_dict = copy.deepcopy(list(xml_dict.values())[0])
    xpath = ""

    return xpath, tmp_dict
